/* Sistemas Integrados TEKNISA
   Compras e Suprimentos

Nome: 8171_CMPSTPS_PERIODICIDADE_NAO_CALCULADA.sql

Descrio:
Calcula o calendrio de compras para as periodicidades que ainda no o possuem

Criao:
04/01/2007 Marcelo Alexsandro Souza Silva
*/

IF EXISTS (SELECT * FROM sysobjects O, sysusers U
          WHERE
            o.name = 'PERIODICIDADE_NAO_CALCULADA'  and
            type = 'P'                  and
            u.uid = o.uid               and
            u.name = USER)
   DROP PROCEDURE PERIODICIDADE_NAO_CALCULADA;
GO

CREATE PROCEDURE PERIODICIDADE_NAO_CALCULADA
   @P_CDFILIAL    VARCHAR(255),
   @P_TCDFILIAL   VARCHAR(1),
   @P_CDGRUPFILI  VARCHAR(10),
   @P_CDPRODINI   VARCHAR(20),
   @P_CDPRODFIN   VARCHAR(20)
AS

DECLARE @V_IDSISDOS      VARCHAR(1);
DECLARE @V_IDUTILDTPROC  VARCHAR(1);
DECLARE @V_CDPRODINI     VARCHAR(40);
DECLARE @V_CDPRODFIN     VARCHAR(40);
DECLARE @VCUR_FILIAL     VARCHAR(4);
DECLARE @V_CDPERIENTR    VARCHAR(2);
DECLARE @V_DTPREVCONS    DATETIME;
DECLARE @V_DTPROCESS     DATETIME;

BEGIN
   SELECT
      @V_IDSISDOS     = MAX(IDSISDOS),
      @V_IDUTILDTPROC = MAX(IDUTILDTPROC)
   FROM
      PARAMGERAL;

   IF @P_CDPRODINI > @P_CDPRODFIN
   BEGIN
      SET @V_CDPRODINI = @P_CDPRODINI + 'ZZZZZZZZZ';
      SET @V_CDPRODFIN = @P_CDPRODFIN;
   END
   ELSE
   BEGIN
      SET @V_CDPRODFIN = @P_CDPRODFIN + 'ZZZZZZZZZ';
      SET @V_CDPRODINI = @P_CDPRODINI;
   END;

   DECLARE CUR_FILIAL CURSOR FOR
      (SELECT
          FILIAL.CDFILIAL
       FROM
          FILIAL
       WHERE
          (CHARINDEX(FILIAL.CDFILIAL, @P_CDFILIAL) > 0 OR @P_TCDFILIAL = 'T') AND
          (@P_CDGRUPFILI IS NULL)
       UNION
       SELECT
          FILGRUPFILI.CDFILIAL
       FROM
          FILGRUPFILI
       WHERE
          (CHARINDEX(FILGRUPFILI.CDFILIAL, @P_CDFILIAL) > 0 OR @P_TCDFILIAL = 'T') AND
          (FILGRUPFILI.CDGRUPFILI = @P_CDGRUPFILI))
   OPEN CUR_FILIAL;
   FETCH NEXT FROM CUR_FILIAL INTO @VCUR_FILIAL;
   WHILE (@@FETCH_STATUS = 0)
   BEGIN
      /* - Verifica a data de processamento para a filial informada ------------ */
      IF @V_IDUTILDTPROC <> 'S'
      BEGIN
         SET @V_DTPROCESS = CONVERT(DATETIME, CONVERT(VARCHAR, GETDATE(), 103), 103);
      END
      ELSE
      BEGIN
         SELECT
            @V_DTPROCESS = ISNULL(MAX(DTPROCESSA),CONVERT(DATETIME, CONVERT(VARCHAR, GETDATE(), 103), 103))
         FROM
            PARAMFILIAL
         WHERE
            (CDFILIAL = @VCUR_FILIAL);
      END;
      /* Calcula calendrio de compras para as periodicidades que ainda no possuem */
      DECLARE C_PERIODNCALC CURSOR FOR
         (SELECT DISTINCT
            PERFILPROD.CDPERIENTR,
            PREVCONGER.DTPREVCONS - DAY(PREVCONGER.DTPREVCONS) + 1 AS DTPREVCONS
          FROM
             (SELECT
               PREVCONS.CDFILIAL,
               CONVERT(DATETIME, CONVERT(VARCHAR, PREVCONS.DTPREVCONS, 103), 103) AS DTPREVCONS,
               PROD_PREV.CDPRODESTO AS CDPRODUTO,
               PROD_PREV.CDARVPROD
             FROM
               PRODUTO PROD_PREV,
               PREVCONS
             WHERE
               PREVCONS.CDFILIAL    = @VCUR_FILIAL  AND
               CONVERT(DATETIME, CONVERT(VARCHAR, PREVCONS.DTPREVCONS, 103), 103) >= @V_DTPROCESS AND
               PREVCONS.CDPRODUTO = PROD_PREV.CDPRODUTO AND
               PROD_PREV.CDARVPROD BETWEEN @V_CDPRODINI AND @V_CDPRODFIN

             UNION ALL

             SELECT
               ITPREVCOMP.CDFILIAL,
               CONVERT(DATETIME, CONVERT(VARCHAR, ITPREVCOMP.DTENTPRECOMP, 103), 103) AS DTPREVCONS,
               PROD_PREV.CDPRODESTO AS CDPRODUTO,
               PROD_PREV.CDARVPROD
             FROM
               PRODUTO PROD_PREV,
               ITPREVCOMP
             WHERE
               ITPREVCOMP.CDFILIAL    = @VCUR_FILIAL  AND
               CONVERT(DATETIME, CONVERT(VARCHAR, ITPREVCOMP.DTENTPRECOMP, 103), 103) >= @V_DTPROCESS AND
               ITPREVCOMP.CDPRODUTO = PROD_PREV.CDPRODUTO    AND
               ITPREVCOMP.IDSTITPRCOMP = 'A'                 AND
               PROD_PREV.CDARVPROD BETWEEN @V_CDPRODINI AND @V_CDPRODFIN

             UNION ALL

             SELECT DISTINCT
                @VCUR_FILIAL AS CDFILIAL,
                CONVERT(DATETIME, CONVERT(VARCHAR, ITEMSOLI.DTPROGENTR, 103), 103) AS DTPREVCONS,
                PRODUTO.CDPRODESTO AS CDPRODUTO,
                PRODUTO.CDARVPROD
             FROM
                ITEMSOLI,
                PRODUTO,
                FILIAL
             WHERE
                FILIAL.CDFILCENTPME = @VCUR_FILIAL AND
                CONVERT(DATETIME, CONVERT(VARCHAR, ITEMSOLI.DTPROGENTR, 103), 103) >= @V_DTPROCESS AND
                ISNULL(ITEMSOLI.IDITMCONS, 'N')  = 'N'       AND
                ISNULL(ITEMSOLI.IDSOLIAPROV, 'S')= 'S'       AND
                PRODUTO.CDPRODUTO = ITEMSOLI.CDPRODUTO  AND
                PRODUTO.CDARVPROD BETWEEN @V_CDPRODINI   AND @V_CDPRODFIN AND
                ITEMSOLI.CDFILSOLI  = FILIAL.CDFILIAL     AND
                ITEMSOLI.CDFILSOLI+ITEMSOLI.CDPRODUTO NOT IN (SELECT
                                                                 CMPCENTENDIR.CDFILIAL+CMPCENTENDIR.CDPRODUTO
                                                              FROM
                                                                 CMPCENTENDIR,
                                                                 PRODUTO PI,
                                                                 PRODUTO PC
                                                              WHERE
                                                                 CMPCENTENDIR.CDFILIAL  = ITEMSOLI.CDFILSOLI AND
                                                                 CMPCENTENDIR.CDPRODUTO = PC.CDPRODUTO AND
                                                                 ITEMSOLI.CDPRODUTO     = PI.CDPRODUTO AND
                                                                 PI.CDARVPROD LIKE PC.CDARVPROD+'%')
             UNION ALL

             SELECT
               PREVCONSPME.CDFILIAL,
               CONVERT(DATETIME, CONVERT(VARCHAR, PREVCONSPME.DTPREVCONPME, 103), 103) AS DTPREVCONS,
               PROD_PREV.CDPRODESTO AS CDPRODUTO,
               PROD_PREV.CDARVPROD
             FROM
               PRODUTO PROD_PREV,
               PREVCONSPME
             WHERE
               PREVCONSPME.CDFILIAL    = @VCUR_FILIAL AND
               CONVERT(DATETIME, CONVERT(VARCHAR, PREVCONSPME.DTPREVCONPME, 103), 103) >= @V_DTPROCESS AND
               PREVCONSPME.CDPRODUTO = PROD_PREV.CDPRODUTO AND
               PROD_PREV.CDARVPROD BETWEEN @V_CDPRODINI AND @V_CDPRODFIN

             UNION ALL

             SELECT
               PREVCOMPRA.CDFILIAL,
               CONVERT(DATETIME, CONVERT(VARCHAR, PREVCOMPRA.DTPREVCOMP, 103), 103) AS DTPREVCONS,
               PROD_PREV.CDPRODESTO AS CDPRODUTO,
               PROD_PREV.CDARVPROD
             FROM
               PRODUTO PROD_PREV,
               PREVCOMPRA
             WHERE
               PREVCOMPRA.CDFILIAL    = @VCUR_FILIAL AND
               CONVERT(DATETIME, CONVERT(VARCHAR, PREVCOMPRA.DTPREVCOMP, 103), 103) >= @V_DTPROCESS AND
               PREVCOMPRA.CDPRODUTO = PROD_PREV.CDPRODUTO AND
               PROD_PREV.CDARVPROD BETWEEN @V_CDPRODINI AND @V_CDPRODFIN

             UNION ALL

             SELECT
               PC.CDFILIAL,
               CONVERT(DATETIME, CONVERT(VARCHAR, PV.DTPREVCONPDV, 103), 103) AS DTPREVCONS,
               PROD_PREV.CDPRODESTO AS CDPRODUTO,
               PROD_PREV.CDARVPROD
             FROM
               PRODUTO PROD_PREV,
               PREVCONSPDV PC,
               PREVISAOPDV PV
             WHERE
               PC.CDFILIAL = PV.CDFILIAL     AND
               PC.NRSEQPREVPDV = PV.NRSEQPREVPDV AND
               PC.CDFILIAL      = @VCUR_FILIAL  AND
               CONVERT(DATETIME, CONVERT(VARCHAR, PV.DTPREVCONPDV, 103), 103) >= @V_DTPROCESS AND
               PC.CDPRODUTO     = PROD_PREV.CDPRODUTO AND
               PROD_PREV.CDARVPROD BETWEEN @V_CDPRODINI AND @V_CDPRODFIN

             UNION ALL

             SELECT
               PREVCONSVMN.CDFILIALCLIE,
               CONVERT(DATETIME, CONVERT(VARCHAR, PREVCONSVMN.DTPREVENDVMN, 103), 103) AS DTPREVCONS,
               PROD_PREV.CDPRODESTO AS CDPRODUTO,
               PROD_PREV.CDARVPROD
             FROM
               PRODUTO PROD_PREV,
               PREVCONSVMN
             WHERE
               PREVCONSVMN.CDFILIALCLIE  = @VCUR_FILIAL  AND
               CONVERT(DATETIME, CONVERT(VARCHAR, PREVCONSVMN.DTPREVENDVMN, 103), 103)  >= @V_DTPROCESS AND
               PREVCONSVMN.CDPRODUTO     = PROD_PREV.CDPRODUTO AND
               PROD_PREV.CDARVPROD BETWEEN @V_CDPRODINI AND @V_CDPRODFIN

             UNION ALL

             SELECT
               FI.CDFILIAL,
               CONVERT(DATETIME, CONVERT(VARCHAR, PC.DTPREVCONS, 103), 103) AS DTPREVCONS,
               PROD_PREV.CDPRODESTO AS CDPRODUTO,
               PROD_PREV.CDARVPROD
             FROM
               PRODUTO PROD_PREV,
               PREVCONS PC,
               FILINECE FI,
               FILIAL
             WHERE
               FI.CDFILINECCMP  = PC.CDFILIAL  AND
               FI.CDFILIAL = FILIAL.CDFILIAL     AND
               ISNULL(FILIAL.IDFILCENTPME, 'N') = 'N'         AND
               FI.IDSOLIPREV    = 'P'          AND
               FI.CDFILIAL      = @VCUR_FILIAL   AND
               CONVERT(DATETIME, CONVERT(VARCHAR, PC.DTPREVCONS, 103), 103) >= @V_DTPROCESS AND
               PC.CDPRODUTO     = PROD_PREV.CDPRODUTO AND
               PROD_PREV.CDARVPROD BETWEEN @V_CDPRODINI AND @V_CDPRODFIN

             UNION ALL

             SELECT
               FI.CDFILIAL,
               CONVERT(DATETIME, CONVERT(VARCHAR, PC.DTPREVCOMP, 103), 103) AS DTPREVCONS,
               PROD_PREV.CDPRODESTO AS CDPRODUTO,
               PROD_PREV.CDARVPROD
             FROM
               PRODUTO PROD_PREV,
               PREVCOMPRA PC,
               FILINECE FI,
               FILIAL
             WHERE
               FI.CDFILINECCMP  = PC.CDFILIAL  AND
               FI.CDFILIAL = FILIAL.CDFILIAL     AND
               ISNULL(FILIAL.IDFILCENTPME, 'N') = 'N'         AND
               FI.IDSOLIPREV    = 'P'          AND
               FI.CDFILIAL      = @VCUR_FILIAL   AND
               CONVERT(DATETIME, CONVERT(VARCHAR, PC.DTPREVCOMP, 103), 103) >= @V_DTPROCESS AND
               PC.CDPRODUTO     = PROD_PREV.CDPRODUTO AND
               PROD_PREV.CDARVPROD BETWEEN @V_CDPRODINI AND @V_CDPRODFIN

             UNION ALL

             SELECT
               FI.CDFILIAL,
               CONVERT(DATETIME, CONVERT(VARCHAR, PV.DTPREVCONPDV, 103), 103) AS DTPREVCONS,
               PROD_PREV.CDPRODESTO AS CDPRODUTO,
               PROD_PREV.CDARVPROD
             FROM
               PRODUTO PROD_PREV,
               PREVCONSPDV PC,
               PREVISAOPDV PV,
               FILINECE FI,
               FILIAL
             WHERE
               PC.CDFILIAL = PV.CDFILIAL    AND
               FI.CDFILIAL = FILIAL.CDFILIAL     AND
               ISNULL(FILIAL.IDFILCENTPME, 'N') = 'N'         AND
               PC.NRSEQPREVPDV = PV.NRSEQPREVPDV AND
               FI.CDFILINECCMP  = PC.CDFILIAL  AND
               FI.IDSOLIPREV    = 'P'          AND
               FI.CDFILIAL      = @VCUR_FILIAL   AND
               CONVERT(DATETIME, CONVERT(VARCHAR, PV.DTPREVCONPDV, 103), 103)  >= @V_DTPROCESS AND
               PC.CDPRODUTO     = PROD_PREV.CDPRODUTO AND
               PROD_PREV.CDARVPROD BETWEEN @V_CDPRODINI AND @V_CDPRODFIN

             UNION ALL

             SELECT
               FI.CDFILIAL,
               CONVERT(DATETIME, CONVERT(VARCHAR, PC.DTPREVENDVMN, 103), 103) AS DTPREVCONS,
               PROD_PREV.CDPRODESTO AS CDPRODUTO,
               PROD_PREV.CDARVPROD
             FROM
               PRODUTO PROD_PREV,
               PREVCONSVMN PC,
               FILINECE FI,
               FILIAL
             WHERE
               FI.CDFILINECCMP  = PC.CDFILIALCLIE AND
               FI.CDFILIAL = FILIAL.CDFILIAL     AND
               ISNULL(FILIAL.IDFILCENTPME, 'N') = 'N'         AND
               FI.IDSOLIPREV    = 'P'             AND
               FI.CDFILIAL      = @VCUR_FILIAL    AND
               CONVERT(DATETIME, CONVERT(VARCHAR, PC.DTPREVENDVMN, 103), 103) >= @V_DTPROCESS    AND
               PC.CDPRODUTO     = PROD_PREV.CDPRODUTO AND
               PROD_PREV.CDARVPROD BETWEEN @V_CDPRODINI AND @V_CDPRODFIN

             UNION ALL

             SELECT
               FI.CDFILIAL,
               IT.DTPROGENTR  AS DTPREVCONS,
               PR.CDPRODESTO  AS CDPRODUTO,
               PR.CDARVPROD
             FROM
               ITEMSOLI IT,
               PRODUTO PR,
               FILINECE FI,
               FILIAL
             WHERE
               FI.CDFILIAL     = @VCUR_FILIAL   AND
               FI.CDFILIAL = FILIAL.CDFILIAL     AND
               ISNULL(FILIAL.IDFILCENTPME, 'N') = 'N'         AND
               IT.DTPROGENTR   >= @V_DTPROCESS AND
               PR.CDARVPROD BETWEEN @V_CDPRODINI AND @V_CDPRODFIN AND
               FI.CDFILINECCMP = IT.CDFILSOLI AND
               FI.IDSOLIPREV  <> 'P'          AND
               IT.CDPRODUTO    = PR.CDPRODUTO) PREVCONGER,
             PERFILPROD,
             PRODUTO PROD_PER
          WHERE
            (PERFILPROD.CDPRODUTO = PROD_PER.CDPRODUTO) AND
            (PERFILPROD.CDFILIAL  = @VCUR_FILIAL) AND
            (PROD_PER.CDARVPROD = (SELECT
                                      MAX(PRODUTO.CDARVPROD)
                                   FROM
                                      PERFILPROD A,
                                      PRODUTO
                                   WHERE
                                      (A.CDPRODUTO = PRODUTO.CDPRODUTO) AND
                                      (A.CDFILIAL = @VCUR_FILIAL) AND
                                      (CHARINDEX(PRODUTO.CDARVPROD, PROD_PER.CDARVPROD) = 1))) AND
           (PERFILPROD.CDPERIENTR NOT IN (SELECT
                                             CDPERIENTR
                                          FROM
                                             CALECOMP
                                          WHERE
                                             (CALECOMP.CDFILIAL = @VCUR_FILIAL) AND
                                             (MONTH(CALECOMP.DTENTREGA) = MONTH(PREVCONGER.DTPREVCONS) AND
                                              YEAR(CALECOMP.DTENTREGA) = YEAR(PREVCONGER.DTPREVCONS))))
      )
      OPEN C_PERIODNCALC;
      FETCH NEXT FROM C_PERIODNCALC INTO @V_CDPERIENTR, @V_DTPREVCONS;
      WHILE (@@FETCH_STATUS = 0)
      BEGIN
         EXEC CALC_DATA_ENTR_MES @VCUR_FILIAL,
                                 @V_CDPERIENTR,
                                 @V_DTPREVCONS,
                                 1,
                                 @V_DTPROCESS;
         FETCH NEXT FROM C_PERIODNCALC INTO @V_CDPERIENTR, @V_DTPREVCONS;
      END;
      FETCH NEXT FROM CUR_FILIAL INTO @VCUR_FILIAL;
      CLOSE C_PERIODNCALC;
      DEALLOCATE C_PERIODNCALC;
   END;
   CLOSE CUR_FILIAL;
   DEALLOCATE CUR_FILIAL;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'PERIODICIDADE_NAO_CALCULADA','5.01.002';
GO

